# $Id: aselect.py 2969 2010-01-25 14:31:42Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-8.5.1/src/delfttilburg/aselect.py $
#!/usr/bin/env python2.5
## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft "Discover" and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008-2009 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008-2009 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

from urllib import urlencode
from cgi import parse_qs
from socket import socket, ssl
from urlparse import urlsplit, urlunsplit
from base64 import b64decode, b64encode
from time import time

class ASelectProfile(object):
    """The A-Select Profile class is a data-containter for the userdata returned
    by the A-Select server when authenticated successfully"""

    def __init__(self, ticket_exp_time=0):
        self.ticket_exp_time = ticket_exp_time

    def isExpired(self):
        """Boolean indicating if the current login has expired"""
        return time() * 1000 > self.ticket_exp_time


class ASelect(object):
    """This module provides access to the following A-Select calls:
    - authenticate
    - verify_credentials
    - kill_tgt
    """

    def __init__(self, aSelectHost='', port=1495, app_id=''):
        """Construct an A-Select object usable in Meresco DNA.
        - aSelectHost: the hostname of the A-Select server/agent
        - port: port on which the A-Select server is reachable
        - app_id: the id of the application as handed out by the A-Select server
                  administrator.
        """
        self._aSelectHost = aSelectHost
        self._port = port
        self._app_id = app_id

    def _sendRequestToASelect(self, parameters):
        s = socket()
        try:
            s.connect((self._aSelectHost, self._port))
            data = urlencode(parameters)
            s.send(data+"\n")
            data = s.recv(4096)
            return parse_qs(data)
        finally:
            s.close()


    def authenticate(self, app_url=''):
        """Perform the 'authenticate' call to an A-Select server.
        If authenticated successfully, the user will be redirected to the supplied app_url."""
        parameters = {
            'request': 'authenticate',
            'app_id': self._app_id,
            'app_url': app_url,
        }

        answer = self._sendRequestToASelect(parameters)
        resultCode = answer['result_code'][0]
        as_url = ''

        if resultCode == '0000':
            scheme, netloc, path, query, fragment = urlsplit(answer['as_url'][0])
            arguments = parse_qs(query)
            newArguments = {}
            newArguments.update(answer)
            newArguments.update(arguments)
            del newArguments['as_url']
            del newArguments['result_code']

            query = urlencode(newArguments, doseq=True)
            as_url = urlunsplit((scheme, netloc, path, query, fragment))
        return (resultCode, as_url)

    def verifyCredentials(self, rid='', aselect_credentials=''):
        """Perform the 'verify_credentials' call to an A-Select server.
        This method will return the result code of the A-Select call and if
        successfull verified, an ASelectProfile object containing the userdata
        as returned by the A-Select server.
        """
        parameters = {
            'request': 'verify_credentials',
            'rid': rid,
            'aselect_credentials': aselect_credentials
        }

        answer = self._sendRequestToASelect(parameters)
        resultCode = answer['result_code'][0]
        if resultCode != '0000':
            return resultCode, None, None

        profile = self._profileFromResult(answer)
        ticket = answer['ticket'][0]

        return resultCode, profile, ticket

    def verifyTicket(self, ticket='', uid='', organization=''):
        parameters = {
            'request': 'verify_ticket',
            'ticket': ticket,
            'uid': uid,
            'organization': organization
        }

        answer = self._sendRequestToASelect(parameters)
        resultCode = answer['result_code'][0]
        return resultCode, None

    def attributes(self, ticket='', uid='', organization=''):
        parameters = {
            'request': 'attributes',
            'ticket': ticket,
            'uid': uid,
            'organization': organization
        }

        answer = self._sendRequestToASelect(parameters)
        resultCode = answer['result_code'][0]
        if resultCode != '0000':
            return resultCode, None

        return resultCode, self._profileFromResult(answer)


    def killTicket(self, ticket=''):
        """Perform the 'kill_ticket' call to an A-Select server. This method
        return the result_code returned by the A-Select server. On success,
        the user has been logged out.
        """
        parameters = {
            'request': 'kill_ticket',
            'ticket': ticket
        }

        answer = self._sendRequestToASelect(parameters)
        return answer['result_code'][0]

    def encodeAttributes(self, **kwargs):
        return b64encode(urlencode(kwargs))

    def decodeAttributes(self, aString):
        return dict((key, values[0]) for key, values in parse_qs(b64decode(aString)).items())

    def _profileFromResult(self, answer):
        attributes = self.decodeAttributes(answer['attributes'][0])

        profile = ASelectProfile()
        for key, method in [('asp', str), ('uid', str), ('authsp_level', int), ('asp_level', int), ('authsp', str), ('ticket_start_time', int), ('ticket_exp_time', int), ('organization', str)]:
            if key in answer:
                setattr(profile, key, method(answer[key][0]))
        for key, value in attributes.items():
            setattr(profile, key, value)

        return profile
